package ui.cursors
{
	import flash.display.DisplayObject;
	import flash.display.DisplayObjectContainer;
	import flash.events.Event;
	import flash.events.IEventDispatcher;
	import flash.events.MouseEvent;
	import flash.ui.Mouse;
	import flash.ui.MouseCursor;

	public class CursorManager
	{
		private static var _owner:DisplayObjectContainer;
		private static var _dispatcher:IEventDispatcher;
		private static var _stack:CursorStack;
		private static var _current:Cursor;
		private static var _defaultSwitcher:Switcher;
		
		/**
		 * @param	owner - контейнер, в который кладётся курсор
		 * @param	dispatcher - источник событий MOUSE_MOVE
		 * (сделано отдельной переменной, т.к. при использовании в качестве
		 * контейнера спрайта, лежащего поверх всего, этот спрайт надо сделать
		 * прозрачным для мыши, а тогда он не будет ловить события MOUSE_MOVE,
		 * т.е. надо в качестве источника события надо использовать другой
		 * объект)
		 */
		public static function init(
			owner:DisplayObjectContainer, dispatcher:IEventDispatcher):void
		{
			_owner = owner;
			_dispatcher = dispatcher;
			
			_stack = new CursorStack();
			_stack.onChange = onStackChange;
		}
		
		/**
		 * Взять _ручной_ переключатель курсора для отдельной части системы,
		 * чтобы испльзовать его не думая о других подсистемах приложения
		 * @param	priority
		 * @return
		 */
		public static function newSwitcher(priority:int = 0):Switcher
		{
			return new Switcher(_stack, priority);
		}
		
		/**
		 * Взять установщик курсора над отображаемыми объектами, чтобы
		 * переключать курсор над объектом независимо от других подсистем
		 * приложения
		 * @param	priority
		 * @return
		 */
		public static function newHover(priority:int = 0):HoverSwitcher
		{
			return new HoverSwitcher(_stack, priority);
		}
		
		/**
		 * Для адептов подхода "Менеджер курсоров должен иметь только
		 * одно поле - курсор".
		 * Использует переключатель с минимальным приоритетом.
		 * Не мешает другим переключателям
		 */
		public static function get defaultCursor():Cursor
		{
			return _defaultSwitcher != null ? _defaultSwitcher.cursor : null;
		}
		public static function set defaultCursor(value:Cursor):void
		{
			if (_defaultSwitcher == null)
			{
				_defaultSwitcher = newSwitcher(int.MIN_VALUE);
				_defaultSwitcher.enabled = true;
			}
			_defaultSwitcher.cursor = value;
		}
		
		private static function onStackChange():void
		{
			var newCursor:Cursor = _stack.cursor;
			if (_current == newCursor)
			{
				return;
			}
			if (_current != null)
			{
				if (_current.view != null)
				{
					_owner.removeChild(_current.view);
					_owner.removeEventListener(Event.ENTER_FRAME, onEnterFrame);
				}
				if (_dispatcher != null)
				{
					_dispatcher.removeEventListener(
						MouseEvent.MOUSE_MOVE, onMouseMove);
				}
				Mouse.show();
				Mouse.cursor = MouseCursor.AUTO;
			}
			_current = newCursor;
			if (_current != null)
			{
				if (_current.view != null)
				{
					_owner.addChild(_current.view);
					_owner.addEventListener(Event.ENTER_FRAME, onEnterFrame);
					updateViewPosition();
				}
				if (_dispatcher != null)
				{
					_dispatcher.addEventListener(
						MouseEvent.MOUSE_MOVE, onMouseMove);
				}
				if (_current.hideSystem)
				{
					Mouse.hide();
				}
				if (_current.systemCursor != null)
				{
					Mouse.cursor = _current.systemCursor;
				}
			}
		}
		
		private static function onEnterFrame(event:Event):void
		{
			updateViewPosition();
		}
		
		private static function onMouseMove(event:MouseEvent):void
		{
			if (_current != null && _current.updateOnMove)
			{
				updateViewPosition();
				event.updateAfterEvent();
			}
		}
		
		private static function updateViewPosition():void
		{
			var view:DisplayObject = _current.view;
			view.x = _owner.mouseX + _current.offsetX;
			view.y = _owner.mouseY + _current.offsetY;
			
			/*
			 * Чтобы если специального спрайта для курсоров не
			 * создавал никто - курсор был поверх
			 */
			var maxIndex:int = _owner.numChildren - 1;
			if (_owner.getChildIndex(view) < maxIndex)
			{
				_owner.setChildIndex(view, maxIndex);
			}
		}
	}
}